/*
 * File         : BV_Message.cpp
 * Author       : Tomasz Stachowiak
 * Created      : Wed May 20 20:19:04 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <BVN/Message.h>
#include <BVN/SOCK_Stream.h>

#include "BV_Msg_Header.h"
#include "BV_Message.h"
#include "BV_Member.h"
#include "BV_Session_Control.h"
#include "BV.h"

BV_Message::BV_Message(SOCK_Stream *stream) : stream_(stream)
{
  body_ = 0;
}

BV_Message::BV_Message(BV_Msg_Header &header) : Message<BV_Msg_Header, SOCK_Stream>(header)
{
}

int BV_Message::execute()
{
  Participant part;
  char host[50];
  struct in_addr addr;
  BV_Member *member;

  int tag = header_.get_tag();
  BV_Session_Control *session_ctrl = BV_Session_Control::instance();
  header_.get_sender(&part);
  addr.s_addr = part.host;
  strcpy(host, inet_ntoa(addr));
  printf("BV_Message: received message of type %d from %s\n", tag, host);

  switch(tag)
    {
    case reqNewRemoteSessionInvite:
      member = new BV_Member(stream_, part.handle, part.user, part.host);
      session_ctrl->invited(member);
      break;
      
    case reqRemoteSessionAccept:
      BVN_DEBUG("BV_Message::execute: invitation accepted\n");
      session_ctrl->accepted(part.user, part.host);
      break;
	
    case reqRemoteSessionReject:
      session_ctrl->rejected(part.user, part.host);
      break;

    case reqRemoteSessionAddMember:
      member = new BV_Member(stream_, part.handle, part.user, part.host);
      session_ctrl->new_member(member);
      break;

    case reqRemoteSessionQuit:
      if(session_ctrl->delete_member(part.handle) < 0)
	BVN_DEBUG("BV_Message::execute: unknown member\n");
      break;

    case reqRemoteSessionAddRest:
      session_ctrl->confirmed(body_);
      break;

    case reqRemoteSessionSendToApp:
      session_ctrl->app_message(part.handle, body_, header_.get_size());
      break;

    case  reqNewRemoteSessionConnect:
      session_ctrl->connect(part.handle, stream_);
      break;

    default:
      break;
    }

  session_ctrl->destroy();
  
  return 0;
}

